﻿using System;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Soap;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;

namespace SerializationEx1
{
    public partial class frmSerialization : Form
    {
        public frmSerialization()
        {
            InitializeComponent();
        }

        private void btnBasicBinarySerialization_Click(object sender, EventArgs e)
        {
            SerializeObject();
            DeSerializeObject();
        }

        private void btnBasicSoapSerialization_Click(object sender, EventArgs e)
        {
            SerializeObjectWithSoap();
            DeSerializeObjectWithSoap();
        }

        private void SerializeObject()
        {
            // Serileştirilebilir tipin instance'ı oluşturuluyor.
            MyClass obj = new MyClass("Private veri", "Public veri");


            // Serileştirilecek nesneyi taşıyacak stream oluşturuluyor.
            // Bu stream System.IO.Stream soyut sınıfından türemiş herhangi 
            // bir stream (MemoryStream, FileStream, NetworkStream) olabilir.
            // Örneğimizde FileStream kullanarak serileştirme işlemi yapacağız.
            Stream stream = new FileStream("SerializedFile.dat", FileMode.Create, FileAccess.Write, FileShare.Write);

            // Nesneyi serileştirmek için BinaryFormatter hazırlanıp hazırlanıp,
            // serileştirme gerçekleştiriliyor.
            System.Runtime.Serialization.IFormatter formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
            formatter.Serialize(stream, obj);

            // Stream kapatılıyor
            stream.Close();
        }

        private void DeSerializeObject()
        {
            // Dosyadan okunup stream'e yazılıyor
            Stream stream = new FileStream("SerializedFile.dat", FileMode.Open, FileAccess.Read, FileShare.Read);

            // BinaryFormatter oluşturuluyor.
            // Serileştirme ve ters- serileştirmede aynı formatter ile işlem yapmamız
            // gerektiğini unutmamak gerekir
            System.Runtime.Serialization.IFormatter formatter = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();

            // Nesne ters-serileştirilip serileştirildiğindeki durumu elde ediliyor.
            // Burada dikkat edilmesi gereken nokta tipin 
            // yüklenicisi (constructor)'nin çağırılmayışıdır.
            MyClass obj = (MyClass)formatter.Deserialize(stream);
            MessageBox.Show(obj.sPublic);

            // Stream kapatılıyor
            stream.Close();
        }

        private void SerializeObjectWithSoap()
        {
            MyClass obj = new MyClass("Private soap veri", "Public soap veri");

            Stream stream = new FileStream("SerializedSoapFile.dat", FileMode.Create, FileAccess.Write, FileShare.Write);

            // Nesneyi serileştirmek için SoapFormatter hazırlanıp hazırlanıp,
            // serileştirme gerçekleştiriliyor.

            System.Runtime.Serialization.IFormatter formatter = new System.Runtime.Serialization.Formatters.Soap.SoapFormatter();
            formatter.Serialize(stream, obj);

            // Stream kapatılıyor
            stream.Close();
        }

        private void DeSerializeObjectWithSoap()
        {
            // Dosyadan okunup stream'e yazılıyor
            Stream stream = new FileStream("SerializedSoapFile.dat", FileMode.Open, FileAccess.Read, FileShare.Read);

            // SoapFormatter oluşturuluyor.
            // Serileştirme ve ters- serileştirmede aynı formatter ile işlem yapmamız
            // gerektiğini unutmamak gerekir
            System.Runtime.Serialization.IFormatter formatter = new System.Runtime.Serialization.Formatters.Soap.SoapFormatter();

            MyClass obj = (MyClass)formatter.Deserialize(stream);
            MessageBox.Show(obj.sPublic);

            // Stream kapatılıyor
            stream.Close();
        }

        private object DeepCloneForSerialization(object obSource)
        {
            // Serileştirilecek nesneyi taşıyacak stream oluşturuluyor.
            Stream stream = new MemoryStream();

            // BinaryFormatter oluşturuluyor.
            BinaryFormatter formatter = new BinaryFormatter();

            // Yukarıdaki açıklamalara istinaden nesne ile nasıl çalışalacı belirtiliyor.
            formatter.Context = new StreamingContext(StreamingContextStates.Clone);

            // Nesne graph'ı memory stream'e serileştiriliyor.
            formatter.Serialize(stream, obSource);

            // Stream pozisyonu başa alınıyor.
            stream.Position = 0;

            stream.Close();

            // Parametre olarak gelen nesnenin kopyası (clone'u) farklı bir nesneye atanıyor.
            object obTarget = formatter.Deserialize(stream);

            return obTarget;
        }

    }


    [Serializable()]
    public class MyClass
    {
        private string sPrivate;
        public string sPublic;

        public MyClass(string arg1, string arg2)
        {
            sPrivate = arg1;
            sPublic = arg2;
        }
    }
}
